clear all; close all;
% Script to analyse abrasion loss data using multiple regression

%
% Import data via File > Import Data... menu option
%
% Code assumes data read in as MATRIX
%
% Using menu options and selecting "Generate MATLAB script" option gives
% following code (I have just changed the filename in the xlsread function):

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Import the file
newData1 = importdata('abrasion.csv');

% Create new variables in the base workspace from those fields.
vars = fieldnames(newData1);
for i = 1:length(vars)
    assignin('base', vars{i}, newData1.(vars{i}));
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%
% Produces matrix plot of data
%
figure;
plotmatrix(data)
title('Matrix plot of Abrasion Loss, Hardness and Tensile Strength','FontSize',16)
orient landscape
print -dpng abrasion.png

%
% Calculate correlation coefficinet
%
corr(data)

%
% Create design matrix X
%
y=data(:,1);
X=[ones(size(data,1),1) data(:,2:end)];

%
% Fit regression model (0.05 specifies 95% confidence level)
%
[B,BINT,R,RINT,STATS] = regress(y,X);

%
% Extract R-squared statistic
%
STATS(1)


% 10-fold Cross-validation of all 3 models of interest

regf = @(Xtrain, ytrain, Xtest)(Xtest * regress(ytrain,Xtrain))

n=size(X,1);% sample size
K=10;% K in K-fold cross validaiton (should be a percentage)
k=floor(n/K) % size of each of the K equal-sized sub-samples
MSE=zeros(size(y));% initialize MSE to 0.0
I=randperm(n);
for cvI=0:K-1
    %I
    Itest = I(cvI*3+1 : cvI*3+3);% indices of test data
    %pause
    Itrain =  [ I( 1:cvI*3 )  I((cvI+1)*3+1 : end)];% indices of training data
    %pause
    
    XTr=X(Itrain,:); %entire design matrix X; change to exclude columns of X
    yTr=y(Itrain,:); % training data for response y
    XTs=X(Itest,:); % test data; change to exclude columns of X
    %(regf(XTr,yTr,XTs)) % this just gives the prediction for the test data
    MSE(Itest,:)=((y(Itest,:) - regf(XTr,yTr,XTs)) .^2);%
end
% sum of the root mean squared error is
disp('The mean squared error from the DIY 10-fold CV is');
mean(MSE)

% the above output should be very close to the following call to crossval
disp('The mean squared error from the built-in 10-fold CV is');
cvMse=crossval('mse',X,y,'predfun',regf,'Kfold',10)

% cvMse=crossval('mse',X,y,'predfun',regf)
% cvMseNoStrength=crossval('mse',X(:,1:2),y,'predfun',regf)
% cvMseHardness=crossval('mse',X(:,[1,3]),y,'predfun',regf)
%  
% [cvMse cvMseNoStrength cvMseHardness]
 
% Adjusted R-squared for all 3 models of interest
% regstats(y,data(:,2:end),'linear',{'adjrsquare'})
% regstats(y,data(:,2),'linear',{'adjrsquare'})
% regstats(y,data(:,3),'linear',{'adjrsquare'})